'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var chalk = _interopDefault(require('chalk'));
require('fs');
var path = _interopDefault(require('path'));
var coffee = _interopDefault(require('@zeekay/rollup-plugin-coffee'));
var stylus = _interopDefault(require('@zeekay/rollup-plugin-stylus'));
var typescript = _interopDefault(require('rollup-plugin-typescript'));
var json = _interopDefault(require('rollup-plugin-json'));
var pug = _interopDefault(require('rollup-plugin-pug'));
var string = _interopDefault(require('rollup-plugin-string'));
var autoprefixer = _interopDefault(require('autoprefixer'));
var cssnano = _interopDefault(require('cssnano'));
var lost = _interopDefault(require('lost'));
var lostStylus = _interopDefault(require('lost-stylus'));
var postcss = _interopDefault(require('poststylus'));
var rupture = _interopDefault(require('rupture'));
var magic = _interopDefault(require('magic-string'));
var builtins = _interopDefault(require('@zeekay/rollup-plugin-node-builtins'));
var fileSize = _interopDefault(require('filesize'));
var gzip = _interopDefault(require('gzip-size'));
var globals = _interopDefault(require('rollup-plugin-node-globals'));
var commonjs = _interopDefault(require('rollup-plugin-commonjs'));
var es3 = _interopDefault(require('rollup-plugin-es3'));
var executable = _interopDefault(require('rollup-plugin-executable'));
var inject = _interopDefault(require('rollup-plugin-inject'));
var legacy = _interopDefault(require('rollup-plugin-legacy'));
var sizes = _interopDefault(require('rollup-plugin-sizes'));
var sourcemaps = _interopDefault(require('rollup-plugin-sourcemaps'));
var shebang = _interopDefault(require('rollup-plugin-shebang'));
var nodeResolve = _interopDefault(require('@zeekay/rollup-plugin-node-resolve'));
var builtinModules = _interopDefault(require('builtin-modules'));
var rollup = require('rollup');

// src/colors.coffee
var gray, white;

gray = function(s) {
  return chalk.gray(s);
};

gray.bold = function(s) {
  return chalk.gray.bold(s);
};

white = function(s) {
  return chalk.white(s);
};

white.bold = function(s) {
  return chalk.white.bold(s);
};

white.dim = function(s) {
  return chalk.white.dim(s);
};

// src/log.coffee
var log, verbose;

verbose = true;

log = function() {
  if (verbose) {
    return console.log.apply(console, arguments);
  }
};

log.warn = function() {
  if (verbose) {
    return console.log.apply(console, arguments);
  }
};

log.error = function() {
  return console.error.apply(console, arguments);
};

log.verbose = function(bool) {
  return verbose = bool;
};

log.gray = function(message) {
  return log(gray(message));
};

log.white = function(message) {
  return log(white(message));
};

log.white.bold = function(message) {
  return log(white.bold(message));
};

log.white.dim = function(message) {
  return log(white.dim(message));
};

var log$1 = log;

// node_modules/es-object-assign/lib/es-object-assign.mjs
// src/index.coffee
var getOwnSymbols;
var objectAssign;
var shouldUseNative;
var toObject;
var slice = [].slice;

getOwnSymbols = Object.getOwnPropertySymbols;

toObject = function(val) {
  if (val === null || val === void 0) {
    throw new TypeError('Object.assign cannot be called with null or undefined');
  }
  return Object(val);
};

shouldUseNative = function() {
  var i, j, k, len, letter, order2, ref, test1, test2, test3;
  try {
    if (!Object.assign) {
      return false;
    }
    test1 = new String('abc');
    test1[5] = 'de';
    if (Object.getOwnPropertyNames(test1)[0] === '5') {
      return false;
    }
    test2 = {};
    for (i = j = 0; j <= 9; i = ++j) {
      test2['_' + String.fromCharCode(i)] = i;
    }
    order2 = Object.getOwnPropertyNames(test2).map(function(n) {
      return test2[n];
    });
    if (order2.join('') !== '0123456789') {
      return false;
    }
    test3 = {};
    ref = 'abcdefghijklmnopqrst'.split('');
    for (k = 0, len = ref.length; k < len; k++) {
      letter = ref[k];
      test3[letter] = letter;
    }
    if (Object.keys(Object.assign({}, test3)).join('') !== 'abcdefghijklmnopqrst') {
      return false;
    }
    return true;
  } catch (error) {
    return false;
  }
};

var index = objectAssign = (function() {
  if (shouldUseNative()) {
    return Object.assign;
  }
  return function() {
    var from, j, k, key, len, len1, ref, source, sources, symbol, target, to;
    target = arguments[0], sources = 2 <= arguments.length ? slice.call(arguments, 1) : [];
    to = toObject(target);
    for (j = 0, len = sources.length; j < len; j++) {
      source = sources[j];
      from = Object(source);
      for (key in from) {
        if (Object.prototype.hasOwnProperty.call(from, key)) {
          to[key] = from[key];
        }
      }
      if (getOwnSymbols) {
        ref = getOwnSymbols(from);
        for (k = 0, len1 = ref.length; k < len1; k++) {
          symbol = ref[k];
          if (Object.prototype.propIsEnumerable.call(from, symbol)) {
            to[symbol] = from[symbol];
          }
        }
      }
    }
    return to;
  };
})();

// package.json
var name = "handroll";
var version = "0.17.13";
var description = "Expertly rolled JavaScript. API + CLI for for bundling opinionated JavaScript with Rollup.js";
var homepage = "http://github.com/zeekay/handroll";
var bugs = "http://github.com/zeekay/handroll/issues";
var keywords = [
	"bundler",
	"bundling",
	"es6",
	"handroll",
	"modules",
	"preprocessor",
	"rollup"
];
var bin = "bin/handroll";
var main = "lib/handroll.cjs.js";
var module$1 = "lib/handroll.es.js";
var files = [
	"bin/handroll",
	"lib/",
	"src/"
];
var engines = {
	node: ">=4.0.0"
};
var scripts = {
	build: "sake build",
	prepublishOnly: "sake build",
	pretest: "sake build",
	test: "sake test"
};
var license = "MIT";
var repository = {
	type: "git",
	url: "http://github.com/zeekay/handroll"
};
var dependencies = {
	"@zeekay/rollup-plugin-coffee": "^0.1.19",
	"@zeekay/rollup-plugin-node-builtins": "^2.1.2",
	"@zeekay/rollup-plugin-node-resolve": "^0.2.13",
	"@zeekay/rollup-plugin-stylus": "^1.3.6",
	autoprefixer: "^8.6.2",
	"browser-resolve": "^1.11.3",
	"builtin-modules": "^3.0.0",
	chalk: "^2.4.1",
	cssnano: "^3.10.0",
	"es-object-assign": "^4.1.4",
	filesize: "^3.6.1",
	"gzip-size": "^4.1.0",
	lost: "^8.3.0",
	"lost-stylus": "^0.1.3",
	"magic-string": "^0.25.0",
	poststylus: "^1.0.0",
	resolve: "^1.10.1",
	rollup: "^0.60.7",
	"rollup-plugin-commonjs": "^9.1.3",
	"rollup-plugin-es3": "^1.1.0",
	"rollup-plugin-executable": "^1.4.5",
	"rollup-plugin-inject": "^2.0.0",
	"rollup-plugin-json": "^3.0.0",
	"rollup-plugin-legacy": "^1.0.0",
	"rollup-plugin-node-globals": "^1.2.1",
	"rollup-plugin-pug": "^0.1.6",
	"rollup-plugin-shebang": "^0.1.5",
	"rollup-plugin-sizes": "^0.4.2",
	"rollup-plugin-sourcemaps": "^0.4.2",
	"rollup-plugin-string": "^2.0.2",
	"rollup-plugin-strip": "^1.1.1",
	"rollup-plugin-typescript": "^0.8.1",
	rupture: "^0.7.1",
	"uglify-es": "^3.3.10"
};
var devDependencies = {
	coffeescript: "^2.4.1",
	"sake-cli": "^0.7.3",
	"sake-outdated": "^0.4.4",
	"sake-publish": "^0.1.17",
	"sake-version": "^0.1.19",
	stylus: "^0.54.5"
};
var pkg = {
	name: name,
	version: version,
	description: description,
	homepage: homepage,
	bugs: bugs,
	keywords: keywords,
	bin: bin,
	main: main,
	module: module$1,
	"jsnext:main": "lib/handroll.es.js",
	files: files,
	engines: engines,
	scripts: scripts,
	license: license,
	repository: repository,
	dependencies: dependencies,
	devDependencies: devDependencies
};

// src/utils.coffee
var hasProp = {}.hasOwnProperty;

// Create a merged copy of a set of objects
var merge = function(fn) {
  return function(opts) {
    opts = index({}, this.opts, opts);
    if (opts.quiet != null) {
      log$1.verbose(!opts.quiet);
    }
    if (opts.input == null) {
      opts.input = opts.entry; // TODO: Get rid of hack
    }
    return fn.call(this, opts);
  };
};

// Try to guess moduleName (used in export for browser bundles)
var moduleName = function(name$$1) {
  var first;
  if (!name$$1.trim()) {
    return '';
  }
  first = name$$1.charAt(0).toUpperCase();
  name$$1 = name$$1.replace(/\.js$|\.coffee$|-js$/, '');
  name$$1 = name$$1.replace(/-([a-z])/g, function(g) {
    return g[1].toUpperCase();
  });
  return first + name$$1.slice(1);
};

var isPlugin = (function() {
  var keys;
  keys = new Set(['name', 'options', 'load', 'resolveId', 'transform', 'transformBundle', 'ongenerate', 'onwrite', 'intro', 'outro', 'banner', 'footer']);
  return function(obj) {
    var k;
    for (k in obj) {
      if (!hasProp.call(obj, k)) continue;
      if (keys.has(k)) {
        return true;
      }
    }
    return false;
  };
})();

var banner = function() {
  if (banner.seen == null) {
    log$1.white.dim(`handroll v${version} 🍣`);
    return banner.seen = true;
  }
};

var isArray = function(v) {
  return Array.isArray(v);
};

var isString = function(v) {
  return typeof v === 'string' || v instanceof String;
};

// src/external.coffee
var detectExternal, logExternals, logIncluded, removeIncluded;

// Get external packages from package.json
var getDeps = function(pkg, dev = false) {
  var deps, devDeps, peerDeps, ref, ref1, ref2;
  deps = Object.keys((ref = pkg.dependencies) != null ? ref : {});
  devDeps = Object.keys((ref1 = pkg.devDependencies) != null ? ref1 : {});
  peerDeps = Object.keys((ref2 = pkg.peerDependencies) != null ? ref2 : {});
  // Should always include peer dependencies, these are not even part of the
  // package being bundled.
  deps = deps.concat(peerDeps);
  if (dev) {
    return deps.concat(devDeps);
  } else {
    return deps;
  }
};

// Convert external opt into detected externals based on option selected
detectExternal = function(external, pkg) {
  if (external === true || (external == null)) {
    external = getDeps(pkg);
  }
  if (external === 'dev') {
    external = getDeps(pkg, true);
  }
  if (external === false) {
    external = [];
  }
  return external;
};

// Remove included deps from detected externals
removeIncluded = function(externals, include = []) {
  var dep, i, idx, len, removed;
  removed = [];
  for (i = 0, len = include.length; i < len; i++) {
    dep = include[i];
    if (~(idx = externals.indexOf(dep))) {
      externals.splice(idx, 1);
      removed.push(dep);
    }
  }
  return removed;
};

// Log detected external deps
logExternals = function(externals) {
  var dep, i, len;
  if (externals.length) {
    log$1('external:');
    for (i = 0, len = externals.length; i < len; i++) {
      dep = externals[i];
      log$1(` - ${dep}`);
    }
  } else {
    log$1('no externals');
  }
};

// Log detected external deps
logIncluded = function(included) {
  var dep, i, len;
  if (included.length) {
    log$1('included:');
    for (i = 0, len = included.length; i < len; i++) {
      dep = included[i];
      log$1(` + ${dep}`);
    }
  }
};

// autoExternal uses your package.json to guess which dependencies should be
// considered external. By default only dependencies in pkg.dependencies will be
// considered external as pkg.devDependencies are generally only required as
// part of a build or development process. If this isn't what you want you
// should manually specify external dependencies instead.
var autoExternal = function({external, include, pkg}) {
  var externals, included;
  if (isArray(external)) {
    return external;
  }
  externals = detectExternal(external, pkg);
  included = removeIncluded(externals, include);
  logExternals(externals);
  logIncluded(included);
  return externals;
};

// src/formats.coffee
var nameFromPkg;

nameFromPkg = function(opts) {
  var ref;
  return moduleName((ref = opts.pkg) != null ? ref.name : void 0);
};

// autoFormat tries to guess the formats required for an operation based on your
// package.json file. If opts.format is specified, multiple formats will not be
// considered.
var autoFormats = function({format, formats, pkg}) {
  if (format != null) {
    return [format];
  } else if (formats != null) {
    return formats;
  } else {
    formats = [];
    if (pkg.browser != null) {
      formats.push('web');
    }
    if (pkg.main != null) {
      formats.push('cjs');
    }
    if (pkg.module != null) {
      formats.push('es');
    }
    return formats;
  }
};

// Detect formats requested
var detectFormats = function(opts) {
  // opts.format overrides opts.formats
  if (opts.format != null) {
    return [opts.format];
  } else if (opts.formats != null) {
    return opts.formats;
  } else {
    return autoFormats(opts);
  }
};

// Convert our formats to Rollup settings
var formatOpts = function(opts) {
  // Default to commonjs module
  if (opts.format == null) {
    opts.format = 'cjs';
  }
  switch (opts.format) {
    case 'amd':
      return amd(opts);
    case 'cjs':
    case 'commonjs':
    case 'node':
      return cjs(opts);
    case 'es':
    case 'module':
    case 'esmodule':
      return es(opts);
    case 'iife':
    case 'web':
      return iife(opts);
    case 'umd':
      return umd(opts);
    case 'cli':
    case 'bin':
    case 'binary':
    case 'executable':
      return cli(opts);
    default:
      throw new Error('Unsupported export format');
  }
};

// Various pre-configurations we support
var amd = function(opts) {
  return {
    file: opts.output,
    format: 'amd',
    sourcemap: opts.sourcemap
  };
};

var cjs = function(opts) {
  var output, ref, ref1, ref2;
  output = (ref = (ref1 = opts.output) != null ? ref1 : (ref2 = opts.pkg) != null ? ref2.main : void 0) != null ? ref : 'index.js';
  return {
    file: output,
    format: 'cjs',
    sourcemap: opts.sourcemap
  };
};

var es = function(opts) {
  var output, ref, ref1, ref2, ref3, ref4;
  output = (ref = (ref1 = (ref2 = opts.output) != null ? ref2 : (ref3 = opts.pkg) != null ? ref3.module : void 0) != null ? ref1 : (ref4 = opts.pkg) != null ? ref4['js:next'] : void 0) != null ? ref : 'index.mjs';
  return {
    file: output,
    format: 'es',
    sourcemap: opts.sourcemap
  };
};

var iife = function(opts) {
  var name, output, ref, ref1;
  name = (ref = opts.name) != null ? ref : nameFromPkg(opts);
  output = (ref1 = opts.output) != null ? ref1 : `${name}.js`.toLowerCase();
  return {
    file: output,
    format: 'iife',
    browser: opts.browser != null ? opts.browser : opts.browser = true,
    external: false,
    name: name,
    sourcemap: opts.sourcemap
  };
};

var umd = function(opts) {
  return {
    file: opts.output,
    format: 'umd',
    sourcemap: opts.sourcemap
  };
};

var cli = function(opts) {
  var output, ref, ref1, ref2;
  output = (ref = (ref1 = opts.output) != null ? ref1 : (ref2 = opts.pkg) != null ? ref2.bin : void 0) != null ? ref : path.join('bin/', (nameFromPkg(opts)).toLowerCase());
  // Sometimes bin is an object, use the first mapping here
  if (!isString(output)) {
    output = output[(Object.keys(output))[0]];
  }
  return {
    executable: true,
    file: output,
    format: 'cjs',
    sourcemap: opts.sourcemap
  };
};

function autoCompilers(opts) {
  var coffeeOpts, compilers, jsonOpts, k, postPlugins, pugOpts, ref, ref1, ref2, ref3, ref4, ref5, stylusOpts, typescriptOpts, v;
  postPlugins = [
    lost(),
    autoprefixer({
      browsers: '> 1%'
    })
  ];
  if (opts.minify) {
    postPlugins.push(cssnano());
  }
  coffeeOpts = index({}, (ref = opts.compilers) != null ? ref.coffee : void 0);
  typescriptOpts = index({}, (ref1 = opts.compilers) != null ? ref1.typescript : void 0);
  jsonOpts = index({}, (ref2 = opts.compilers) != null ? ref2.json : void 0);
  pugOpts = index({}, {
    compileDebug: false,
    inlineRuntimeFunctions: false,
    pretty: opts.minify ? false : true,
    sourceMap: opts.sourcemap,
    staticPattern: /\S/
  }, (ref3 = opts.compilers) != null ? ref3.pug : void 0);
  stylusOpts = index({}, {
    sourcemap: opts.sourcemap,
    plugins: [lostStylus(), rupture(), postcss(postPlugins)]
  }, (ref4 = opts.compilers) != null ? ref4.stylus : void 0);
  // Default compilers
  compilers = {
    // js:     buble   jsOpts

    // Simple string compilers for .css and .html
    coffee: coffee(coffeeOpts),
    css: string({
      include: '**/*.css'
    }),
    html: string({
      include: '**/*.html'
    }),
    json: json(jsonOpts),
    pug: pug(pugOpts),
    stylus: stylus(stylusOpts),
    typescript: typescript(typescriptOpts)
  };
  compilers.css.name = 'string-css';
  compilers.html.name = 'string-html';
  compilers.typescript.name = 'typescript';
  ref5 = opts.compilers;
  // If passed a legitimate plugin, allow it to override default
  for (k in ref5) {
    v = ref5[k];
    if (isPlugin(v)) {
      compilers[k] = v;
    }
  }
  return compilers;
}

// src/plugins/annotate.coffee

function annotate(opts = {}) {
  return {
    name: 'annotate',
    transform: function(source, id) {
      var filename, ms, result;
      filename = path.relative(process.cwd(), id);
      ms = (new magic(source)).prepend(`// ${filename}\n`);
      result = {
        code: ms.toString()
      };
      if (opts.sourcemap) {
        result.map = ms.generateMap({
          hires: true
        });
      }
      return result;
    }
  };
}

// src/plugins/builtins.coffee

function builtins$1(opts = {}) {
  var plugin;
  plugin = builtins(opts);
  plugin.name = 'node-builtins';
  return plugin;
}

// src/plugins/filesize.coffee
var render;

render = function(filename, size, gzipSize) {
  var gb, wb;
  gb = chalk.green.bold;
  wb = chalk.white.bold;
  return ` ⇢ ${wb(filename)}\t${gb(size)} (${gb(gzipSize)} compressed)`;
};

function filesize(opts = {}) {
  if (opts.format == null) {
    opts.format = {};
  }
  return {
    name: 'filesize',
    ongenerate: function(bundle, result) {
      var gzipSize, size;
      size = fileSize(Buffer.byteLength(result.code, opts.format));
      gzipSize = fileSize(gzip.sync(result.code), opts.format);
      return console.log(render(bundle.file, size, gzipSize));
    }
  };
}

// src/plugins/globals.coffee

function globals$1(opts = {}) {
  var plugin;
  plugin = globals(opts);
  plugin.name = 'node-globals';
  return plugin;
}

// src/plugins/minify.coffee
function minify(opts = {}) {
  var minify;
  ({minify} = require('uglify-es'));
  return {
    name: 'minify',
    transformBundle: function(code, plugins, sourceMapChain, options) {
      var ref, ref1, ref10, ref2, ref3, ref4, ref5, ref6, ref7, ref8, ref9, result;
      result = minify(code, {
        compress: (ref = opts.compress) != null ? ref : {},
        ie8: (ref1 = opts.ie8) != null ? ref1 : false,
        keep_fnames: (ref2 = (ref3 = opts.keepFnames) != null ? ref3 : opts.keep_fnames) != null ? ref2 : false,
        mangle: (ref4 = opts.mangle) != null ? ref4 : {},
        output: (ref5 = opts.output) != null ? ref5 : {},
        parse: (ref6 = opts.parse) != null ? ref6 : {},
        sourcemap: (ref7 = opts.sourcemap) != null ? ref7 : false,
        toplevel: (ref8 = opts.toplevel) != null ? ref8 : false,
        warnings: (ref9 = opts.warnings) != null ? ref9 : false,
        wrap: (ref10 = opts.wrap) != null ? ref10 : false
      });
      return result;
    }
  };
}

// src/plugins/index.coffee
var resolveId;

resolveId = function(id, opts = {}) {
  if (opts.browser) {
    return nodeResolve.browser.sync(id, {
      basedir: opts.basedir
    });
  } else {
    return nodeResolve.node.sync(id, {
      basedir: opts.basedir
    });
  }
};

var autoPlugins = function(opts) {
  var i, k, len, name, plugin, plugins, ref, ref1, ref2, ref3, ref4, ref5, v;
  // Start with source map support
  plugins = [sourcemaps()];
  // Detect compiler plugins or used passed plugins
  if (opts.plugins != null) {
    plugins = plugins.concat(opts.plugins);
  } else {
    ref = autoCompilers(opts);
    for (k in ref) {
      v = ref[k];
      plugins.push(v);
    }
  }
  // Load up any extra plugins specified
  plugins = plugins.concat((ref1 = opts.use) != null ? ref1 : []);
  // Add extra info above each module in bunlde
  plugins.push(annotate({
    sourcemap: opts.sourcemap
  }));
  // Enable legacy
  if (opts.legacy != null) {
    ref2 = opts.legacy;
    for (k in ref2) {
      v = ref2[k];
      try {
        // Attempt to automatically resolve path to legacy files
        opts.legacy[resolveId(k, opts)] = v;
      } catch (error) {
      }
    }
    plugins.push(legacy(opts.legacy));
  }
  // Enable node builtins in browser
  if (opts.builtins || opts.browser && opts.builtins !== false) {
    plugins.push(builtins$1());
  }
  // Automatically resolve node modules
  plugins.push(nodeResolve({
    basedir: opts.basedir,
    browser: opts.browser,
    extensions: opts.extensions,
    preferBuiltins: opts.preferBuiltins,
    external: (ref3 = opts.autoExternal) != null ? ref3 : true,
    skip: (ref4 = opts.skip) != null ? ref4 : opts.external
  }));
  // Enable node globals in browser
  if (opts.globals || opts.browser && opts.globals !== false) {
    plugins.push(globals$1());
  }
  // Slightly more ES3 compatible output
  if (opts.es3 || opts.browser && opts.es3 !== false) {
    plugins.push(es3());
  }
  // Enable CommonJS
  if (opts.commonjs) {
    plugins.push(commonjs(index({
      extensions: opts.extensions,
      sourceMap: opts.sourcemap
    }, opts.commonjs)));
  }
  // Inject imports
  if (opts.inject != null) {
    plugins.push(inject(opts.inject));
  }
  // Automatically make bundle executable
  if (opts.executable) {
    plugins.push(shebang());
    plugins.push(executable());
  }
  // Strip any debugging logic
  if (opts.strip) {
    plugins.push(strip({
      debugger: true,
      functions: ['console.log', 'assert.*', 'debug', 'alert'],
      sourcemap: opts.sourcemap
    }));
  }
  if ((opts.minify != null) && opts.minify !== false) {
    plugins.push(minify(index({}, source({
      ap: opts.sourcemap
    }), opts.minify)));
  }
  // Extra logging + details
  if (!opts.quiet) {
    plugins.push(filesize());
    if (opts.details) {
      plugins.push(sizes({
        details: true
      }));
    }
  }
  log$1('plugins:');
  for (i = 0, len = plugins.length; i < len; i++) {
    plugin = plugins[i];
    name = ((ref5 = plugin.name) != null ? ref5 : '').replace(/rollup-plugin-/, '');
    log$1(` + ${name}`);
  }
  return plugins;
};

// src/generate.coffee

var generate = function(bundle, opts) {
  return bundle.generate(formatOpts(opts));
};

// Helper to write bundle
var write = function(bundle, opts) {
  opts = index({}, opts);
  // Don't pass our specialized options to rollup
  delete opts.executable;
  delete opts.moduleName;
  return bundle.write(formatOpts(opts));
};

// Helper to write multiple formats
var writeAll = function(bundle, opts) {
  var fmt, i, len, ps, ref;
  ps = [];
  ref = detectFormats(opts);
  for (i = 0, len = ref.length; i < len; i++) {
    fmt = ref[i];
    ps.push(write(bundle, index({}, opts, {
      format: fmt
    })));
  }
  return Promise.all(ps);
};

// src/bundle.coffee
var Bundle, cached, readPkg;

cached = null;

readPkg = function() {
  try {
    return require(path.join(process.cwd(), 'package.json'));
  } catch (error) {
    return {};
  }
};

Bundle = (function() {
  class Bundle {
    constructor(opts = {}) {
      var ref;
      if (!(this instanceof Bundle)) {
        return new Bundle(opts);
      }
      if (opts.pkg == null) {
        opts.pkg = readPkg();
      }
      if (opts.acorn == null) {
        opts.acorn = {
          allowReserved: true
        };
      }
      if (opts.browser == null) {
        opts.browser = false;
      }
      if (opts.compilers == null) {
        opts.compilers = null;
      }
      if (opts.es3 == null) {
        opts.es3 = false;
      }
      if (opts.executable == null) {
        opts.executable = false;
      }
      if (opts.extensions == null) {
        opts.extensions = ['.js', '.json', '.coffee', '.pug', '.styl'];
      }
      if (opts.sourcemap == null) {
        opts.sourcemap = true;
      }
      if (opts.external == null) {
        opts.external = null;
      }
      if (opts.include == null) {
        opts.include = [];
      }
      if (opts.inject == null) {
        opts.inject = null;
      }
      if (opts.plugins == null) {
        opts.plugins = null;
      }
      if (opts.use == null) {
        opts.use = [];
      }
      log$1.verbose(!((ref = opts.quiet) != null ? ref : false));
      this.opts = opts;
    }

    cache({cache, invalidate}) {
      var i, id, len;
      if (cache === false) {
        return null;
      }
      if (cache == null) {
        cache = cached;
      }
      if (invalidate != null) {
        for (i = 0, len = invalidate.length; i < len; i++) {
          id = invalidate[i];
          log$1(`removing ${id} from module cache`);
          delete cache[id];
        }
      }
      return cache;
    }

  }
  Bundle.prototype.rollup = merge(function(opts) {
    var k, ref, ref1, ref2, v;
    if (!((opts.input != null) && opts.input !== '')) {
      throw new Error('No input module specified');
    }
    banner();
    if (this.bundle != null) {
      log$1('using cached bundle');
      return Promise.resolve(this.bundle);
    }
    opts.autoExternal = (ref = opts.autoExternal) != null ? ref : opts.external === true;
    opts.basedir = (ref1 = opts.basedir) != null ? ref1 : path.dirname(opts.input);
    // Detect format and update opts accordingly
    opts.formats = autoFormats(opts);
    if (opts.formats.length === 1) {
      opts.format = opts.formats[0];
    }
    // Set a few overrides if a single format is selected
    if (opts.format != null) {
      ref2 = formatOpts(opts);
      for (k in ref2) {
        v = ref2[k];
        opts[k] = v;
      }
    }
    opts.external = autoExternal(opts);
    opts.plugins = autoPlugins(opts);
    return new Promise((resolve, reject) => {
      return rollup.rollup({
        input: opts.input,
        cache: this.cache(opts),
        acorn: opts.acorn,
        external: opts.external,
        plugins: opts.plugins,
        onwarn: function(warning) {
          // Skip built-in module warnings
          if (warning.code === 'UNRESOLVED_IMPORT') {
            if (~builtinModules.indexOf(warning.source)) {
              return;
            }
          }
          if (opts.onwarn != null) {
            // Delegate to onwarn handler
            return opts.onwarn(warning);
          }
          return log$1.warn(warning.message);
        }
      }).then((bundle) => {
        if (opts.cacheBundle) {
          this.bundle = bundle;
        }
        resolve(bundle);
        return log$1.white.bold(opts.input);
      }).catch((err) => {
        var ref3;
        if (((ref3 = err.loc) != null ? ref3.file : void 0) != null) {
          log$1(`\nFailed to parse '${err.loc.file}'`);
          if (err.frame != null) {
            log$1(`\n${err.frame}\n`);
          }
        } else if ((err.plugin != null) && (err.id != null)) {
          log$1(`\nPlugin '${err.plugin}' failed on module ${err.id}`);
          log$1(err.stack);
        } else if (err.id != null) {
          log$1(`\nFailed to parse module ${err.id}`);
          log$1(err.stack);
        } else {
          log$1(err.stack);
        }
        return reject(err);
      });
    });
  });

  Bundle.prototype.generate = merge(function(opts) {
    return new Promise((resolve, reject) => {
      return this.rollup(opts).then(function(bundle) {
        return resolve(generate(bundle, opts));
      }).catch(reject);
    });
  });

  Bundle.prototype.write = merge(function(opts) {
    return new Promise((resolve, reject) => {
      return this.rollup(opts).then(function(bundle) {
        if (opts.format != null) {
          return resolve(write(bundle, opts));
        } else {
          return resolve(writeAll(bundle, opts));
        }
      }).catch(reject);
    });
  });

  return Bundle;

}).call(undefined);

var Bundle$1 = Bundle;

// src/handroll.coffee
var Handroll, sourceMapOverride;

sourceMapOverride = function() {
  if (process.env.DISABLE_SOURCEMAP) {
    return false;
  }
  if (process.env.NO_SOURCEMAP) {
    return false;
  }
  if (process.env.SOURCEMAP) {
    return true;
  }
  return null;
};

Handroll = (function() {
  class Handroll {
    constructor(opts = {}) {
      var ref, ref1;
      if (!(this instanceof Handroll)) {
        return new Handroll(opts);
      }
      if (opts.sourcemap == null) {
        opts.sourcemap = (ref = sourceMapOverride()) != null ? ref : true;
      }
      log$1.verbose(!((ref1 = opts.quiet) != null ? ref1 : false));
      this.opts = opts;
    }

    use(plugin) {
      var i, len, plugins;
      if (isArray(plugin)) {
        plugins = plugin;
      } else {
        plugins = [plugin];
      }
      for (i = 0, len = plugins.length; i < len; i++) {
        plugin = plugins[i];
        this.opts.use.push(plugin);
      }
      return this;
    }

  }
  Handroll.prototype.bundle = merge(function(opts) {
    var bundle;
    bundle = new Bundle$1(opts);
    return new Promise(function(resolve, reject) {
      return (bundle.rollup({
        cacheBundle: true
      })).then(function() {
        return resolve(bundle);
      });
    });
  });

  Handroll.prototype.generate = merge(function(opts) {
    return (new Bundle$1(opts)).generate(opts);
  });

  Handroll.prototype.write = merge(function(opts) {
    return (new Bundle$1(opts)).write(opts);
  });

  return Handroll;

}).call(undefined);

var Handroll$1 = Handroll;

// src/index.coffee
var handroll;

handroll = new Handroll$1();

handroll.Bundle = Bundle$1;

handroll.Handroll = Handroll$1;

handroll.verbose = log$1.verbose;

handroll.version = pkg.version;

var handroll$1 = handroll;

module.exports = handroll$1;
//# sourceMappingURL=handroll.cjs.js.map
